/*---------------------------------------------------------------------------
  This code is part of the book
  LOGIC PROGRAMMING TECHNIQUES: THE PROLOG LANGUAGE
  Ilias Sakellariou, Nikos Bassiliades, Petros Kefalas, Demosthenes Stamatis
  Hellenic Academic On-line Publications, 2015
----------------------------------------------------------------------------*/


%%% 

%%% ECLiPSe Prolog Libraries Required.
:-use_module(library(ic)).
:-use_module(library(branch_and_bound)). 
:-use_module(library(ic_edge_finder)).

%%% car/3
%%% car(Name, Weight, Speed) 
%%% Facts that store information regarding
%%% the trucks that have to cross the bridge.

car(alpha, 10, 4).
car(beta, 13,5).
car(gamma, 8, 3).
car(delta, 5, 4).
car(ephilon, 7, 1).
car(zita, 9, 3).
car(eta, 11, 6).


%%% car_crossing/2
%%% car_crossing(Starts,MakeSpan)
%%% Succeeds if Starts are the crossing times of the trucks
%%% and MakeSpan the makespan of the schedule (completion time of the 
%%% last crossing.

car_crossing(Starts,MakeSpan):-
	findall(C,car(C,_,_),Cars),
	findall(W,car(_,W,_),Weights),
	findall(S,car(_,_,S),Speeds),
	length(Cars,N),
	length(Starts,N), %% My vars
	Starts #:: 0..inf, 	

	state_crossing_times(Starts,Speeds,Ends),

	maxlist(Ends,MakeSpan),
	cumulative(Starts,Speeds,Weights,20),
	bb_min(labeling(Starts),MakeSpan,bb_options{strategy:restart}),
	pretty_print(Cars,Starts,Ends).
	 

%%% state_crossing_times/3
%%% state_crossing_times(Starts,Speeds,Ends)
%%% Imposes the constraints S + Speed #= E for every truck.
state_crossing_times([],[],[]).
state_crossing_times([S|Starts],[Sp|Speeds],[E|Ends]):-
	S + Sp #= E, 	
	state_crossing_times(Starts,Speeds,Ends). 

%%% pretty_print/3
%%% pretty_print(Cars,Starts,Ends)
%%% Pretty prints the 
pretty_print([],[],[]).
pretty_print([C|Cars],[S|Starts],[E|Ends]):-
	write([car,C,leaves,S,until,E]),nl,
	pretty_print(Cars,Starts,Ends).
