
/*---------------------------------------------------------------------------
  This code is part of the book
  LOGIC PROGRAMMING TECHNIQUES: THE PROLOG LANGUAGE
  Ilias Sakellariou, Nikos Bassiliades, Petros Kefalas, Demosthenes Stamatis
  Hellenic Academic On-line Publications, 2015
----------------------------------------------------------------------------*/

%%% loading the ECLiPSe libraries
:-lib(ic).

%%% N Queens Problem
%%% queens/3
%%% queens(N,QRows,Strategy)
%%% Solves the Nqueens problem of a chessboard size NxN, returning
%%% the row positions of the queens list QRows, using 
%%% Strategy (labeling, [first_fail,indomain], etc).

queens(N,QRows,Strategy):-
	length(QRows,N),        
	QRows #:: 1..N,
        alldifferent(QRows), %% different row
	apply_constraints(QRows),
        search_for_solution(QRows,Strategy).

%%% search_for_solution/2
%%% search_for_solution(QRows,Strategu)
%%% Assigns to the list of constraint variables QRows values using the 
%%% Strategy specified.
%%% You can try different heurstics on variable and value ordering.
search_for_solution(QRows,labeling):-
	labeling(QRows).

search_for_solution(QRows,[VarOrdering,ValueOrdering]):-
	search(QRows,0,VarOrdering,ValueOrdering,complete, []).

%%% apply_constraints/1
%%% apply_constraints(QueensR)
%%% States the constraints of the NQueens problem on the list of variables 
%%% QueensR
apply_constraints([]).
apply_constraints([QueenR|Rest]):-
        safe(QueenR,Rest,1),
        apply_constraints(Rest).
%%% safe/3
%%% safe(QueenR,RestQueensR,Pos)
%%% Ensures that a single queen (QueenR) is safe byu posing 
%%% all the constraints with the queens in list RestQueensR.
safe(_,[],_).
safe(QueenR,[NextQueenR|Rest],Pos):-
        NextQueenR #\= QueenR+Pos,
	NextQueenR #\= QueenR-Pos,
        NewPos is Pos + 1,
        safe(QueenR,Rest,NewPos).





